import speech_recognition as sr

def recognize_from_google(voice="voice.wav", lang='en'):
    engine = sr.Recognizer()
    with sr.AudioFile(voice) as source:
        voice_data = engine.record(source)
        try:
            text = engine.recognize_google(voice_data, language=lang) 
            return text
        except sr.UnknownValueError as e:
            return str(e)
        except sr.RequestError as e:
            return str(e)
        
def recognize(voice="voice.wav", lang='en'):
    return recognize_from_google(voice)



from . import Recorder
from . import Recognizer
names = [
    "wiseman"
]

__lissening = True

def lissen(function, recordtime=3):
    global __lissening
    __lissening = True
    while __lissening:
        Recorder.record(recordtime,"voice.wav")
        text = Recognizer.recognize().lower()
        for name in names:
            replacedtext = text.replace(name, '')
            if name in text:
                function(name, replacedtext)
def stop_listen():
    global __lissening
    __lissening = False



from elevenlabs import play
from elevenlabs.client import ElevenLabs
import pygame
from gtts import gTTS
import os

class ElevenLabsApi_info:
    api_key=""
    voice="D1xRw7f8ZHedI7xJgfvz"
    model="eleven_multilingual_v2"

def play_sound(file_path):
    pygame.init()
    pygame.mixer.init()
    pygame.mixer.music.load(file_path)
    pygame.mixer.music.play()

def speak_with_eleven_labs(self,text):
    client = ElevenLabs(api_key=ElevenLabsApi_info.api_key)

    audio = client.generate(
    text= text,
    voice=ElevenLabsApi_info.voice,
    model=ElevenLabsApi_info.model
    )
    play(audio)


def speak_with_gtts(text,lang='en'):
    tts = gTTS(text=text, lang=lang)
    tts.save("saying.mp3")
    play_sound("saying.mp3")    
    os.remove("saying.mp3")

class speak_methods:
    gtts = speak_with_gtts
    elevenlabs = speak_with_eleven_labs

speak_method=speak_methods.gtts

def speak(text,lang='en',speak_method=speak_method):
    speak_method(text,lang)    


import pyaudio
import wave

def record(recordtime, filename):
    CHUNK = 1024
    FORMAT = pyaudio.paInt16
    CHANNELS = 2
    RATE = 44100
    RECORD_SECONDS = recordtime

    audio = pyaudio.PyAudio()
    stream = audio.open(format=FORMAT, channels=CHANNELS,
                        rate=RATE, input=True,
                        frames_per_buffer=CHUNK)

    frames = []

    for _ in range(0, int(RATE / CHUNK * RECORD_SECONDS)):
        frames.append(stream.read(CHUNK))

    stream.stop_stream()
    stream.close()
    audio.terminate()

    with wave.open(filename, 'wb') as wf:
        wf.setnchannels(CHANNELS)
        wf.setsampwidth(audio.get_sample_size(FORMAT))
        wf.setframerate(RATE)
        wf.writeframes(b''.join(frames))